import type { JsonRpcProvider } from '../providers/JsonRpcProvider';
import type { ContractInterface } from '../types/Contract.types';
export declare class BaseContract {
    /**
     * The URL to your Eth node. Consider POKT or Infura
     */
    private readonly _address;
    private readonly _provider;
    /**
     * @param addressOrName The ethereum address of the smart-contract
     * @param contractInterface The JSON ABI of the smart-contract (like http://api.etherscan.io/api?module=contract&action=getabi&address=0x090d4613473dee047c3f2706764f49e0821d256e&format=raw)
     * @param signerOrProvider An instantiated essential-eth provider
     * @example
     */
    constructor(addressOrName: string, contractInterface: ContractInterface, signerOrProvider: JsonRpcProvider);
}
/**
 * Applies the unique contract's methods to the instantiated Contract in the constructor based-upon the provided ABI
 *
 * @param object
 * @param name
 * @param value
 * @internal
 * @example
 */
export declare function defineReadOnly<T>(object: T, name: string, value: any): void;
/**
 * @alpha
 * Only accepts ABIS in JSON format. This allows for stronger typing and assurances of data-types
 * Only read-only function calls currently supported.
 * @example
 * ```typescript
 * import { Contract, JsonRpcProvider } from 'essential-eth';
 * // UNI airdrop contract
 * const contractAddress = '0x090D4613473dEE047c3f2706764f49E0821D256e';
 * const provider = new JsonRpcProvider();
 * // for more robust contract calls, provide a fallback:
 * // const provider = new FallthroughProvider(['bad', 'https://free-eth-node.com/api/eth']);
 *
 * const JSONABI = [
 *   {
 *     inputs: [
 *       {
 *         internalType: 'uint256',
 *         name: 'index',
 *         type: 'uint256',
 *       },
 *     ],
 *     name: 'isClaimed',
 *     outputs: [
 *       {
 *         internalType: 'bool',
 *         name: '',
 *         type: 'bool',
 *       },
 *     ],
 *     stateMutability: 'view',
 *     type: 'function',
 *   },
 * ]
 *
 * const contract = new Contract(
 *   contractAddress,
 *   JSONABI,
 *   provider,
 * );
 *
 * (async () => {
 *   // prints boolean as to whether index 0 has claimed airdrop or not
 *   console.log(await contract.isClaimed(0));
 * })()
 *
 *
 * ```
 */
export declare class Contract extends BaseContract {
    /**
     * The function names on any given contract. Like "isClaimed", "merkleRoot", etc.
     */
    readonly [key: string]: any;
}
