"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeRPCResponse = exports.encodeData = exports.hexFalse = void 0;
const sha3_1 = require("sha3");
const tiny_big_1 = require("../../shared/tiny-big/tiny-big");
const to_checksum_address_1 = require("../../utils/to-checksum-address");
const hex_to_decimal_1 = require("./hex-to-decimal");
exports.hexFalse = '0'.repeat(64);
const hexTrue = '0'.repeat(63) + '1';
/**
 * @param hex
 * @example
 */
function hexToUtf8(hex) {
    let str = '';
    let i = 0;
    const l = hex.length;
    if (hex.substring(0, 2) === '0x') {
        i = 2;
    }
    for (; i < l; i += 2) {
        const code = parseInt(hex.substr(i, 2), 16);
        if (code === 0)
            continue; // Skip null bytes
        str += String.fromCharCode(code);
    }
    try {
        return decodeURIComponent(escape(str)); // Convert UTF-8 to Unicode
    }
    catch (e) {
        return str; // Return original string if conversion fails
    }
}
/**
 * Expands an integer type to use a default of 256 bits. Used for consistency; not required in Solidity
 *
 * @see https://ethereum.stackexchange.com/questions/43241/why-write-uint256-instead-of-uint-if-theyre-the-same-thing
 * @param type the type to explicitly define as 256 bits
 * @returns the integer type expanded to explicitly be 256 bits when possible
 * @example
 * ```javascript
 * expandType('uint[]');
 * // 'uint256[]'
 * ```
 */
function expandType(type) {
    // https://docs.soliditylang.org/en/v0.8.7/types.html#integers
    if (type === 'uint[]') {
        return 'uint256[]';
    }
    else if (type === 'int[]') {
        return 'int256[]';
    }
    return type;
}
/**
 *
 * @param jsonABIArgument
 * @param args
 * @example
 */
function encodeData(jsonABIArgument, args) {
    const hash = new sha3_1.Keccak(256);
    /* first 4 bytes will create the data parameter */
    const functionString = `${jsonABIArgument.name}(${jsonABIArgument.inputs.map((input) => expandType(input.type))})`;
    // encoding learnt from https://ethereum.stackexchange.com/questions/3514/how-to-call-a-contract-method-using-the-eth-call-json-rpc-api
    const functionHash = hash.update(functionString).digest('hex');
    // no arrays
    const jsonABIInputsLength = jsonABIArgument.inputs.length;
    let shouldValidateInputLength = true;
    // inputs contains 1 or more arrays
    if (jsonABIArgument.inputs.find((input) => input.type.includes('['))) {
        shouldValidateInputLength = false;
    }
    if (shouldValidateInputLength && args.length !== jsonABIInputsLength) {
        throw new Error(`args inputs  of "${args.length}" does not match expected length of "${jsonABIArgument.inputs.length}"`);
    }
    const argsWithTypes = (jsonABIArgument.inputs || []).reduce((acc, input, i) => {
        if (input.type.includes('[')) {
            // strip array and length like "[2]" from type
            const basicType = /([^[]*)\[.*$/g.exec(input.type)?.[1];
            args.forEach((arg) => {
                acc = acc.concat([[arg, basicType]]);
            });
            return acc;
        }
        else {
            return acc.concat([[args[i], input.type]]);
        }
    }, []);
    const encodedArgs = argsWithTypes.map(([arg, inputType]) => {
        let rawArg = arg;
        switch (inputType) {
            case 'bool':
                return arg ? hexTrue : exports.hexFalse;
            case 'address':
                // remove leading "0x"
                rawArg = arg.replace(/^0x/g, '').toLowerCase();
                break;
            default:
                if (inputType.startsWith('bytes')) {
                    // encode each character to hex
                    const argEncoded = rawArg
                        .split('')
                        .map((character) => character.charCodeAt(0).toString(16))
                        .join('');
                    const paddedEncodedArg = argEncoded.padEnd(64, '0');
                    return paddedEncodedArg;
                }
                else if (inputType === 'uint256') {
                    const argEncoded = BigInt(arg).toString(16);
                    const paddedEncodedArg = argEncoded.padStart(64, '0');
                    return paddedEncodedArg;
                }
                else if (inputType.startsWith('uint')) {
                    break;
                }
                else {
                    throw new Error(`essential-eth does not yet support "${inputType}" inputs. Make a PR today!"`);
                }
        }
        const argEncoded = rawArg.toString(16);
        const paddedEncodedArg = argEncoded.padStart(64, '0');
        return paddedEncodedArg;
    });
    const functionEncoded = functionHash.slice(0, 8);
    const data = `0x${functionEncoded}${encodedArgs.join('')}`;
    return data;
}
exports.encodeData = encodeData;
/**
 * @internal
 * @param jsonABIArgument
 * @param nodeResponse
 * @example
 */
function decodeRPCResponse(jsonABIArgument, nodeResponse) {
    const rawOutputs = jsonABIArgument.outputs;
    const slicedResponse = nodeResponse.slice(2);
    if (jsonABIArgument?.outputs?.length === 1 &&
        jsonABIArgument.outputs[0].type === 'string') {
        const [hexOffset, responseData] = [
            slicedResponse.slice(0, 64),
            slicedResponse.slice(64),
        ];
        const decimalOffset = Number((0, hex_to_decimal_1.hexToDecimal)(`0x${hexOffset}`));
        const hexLength = responseData.slice(0, decimalOffset * 2);
        const decimalLength = Number((0, hex_to_decimal_1.hexToDecimal)(`0x${hexLength}`));
        const hexToDecode = responseData.slice(decimalOffset * 2, decimalOffset * 2 + decimalLength * 2);
        return hexToUtf8(hexToDecode);
    }
    // chunk response every 64 characters
    const encodedOutputs = slicedResponse.match(/.{1,64}/g);
    const outputs = (encodedOutputs || []).map((output, i) => {
        const outputType = (rawOutputs || [])[i].type;
        switch (outputType) {
            case 'bool':
                return output === hexTrue;
            case 'address':
                /* address types have 24 leading zeroes to remove */
                return (0, to_checksum_address_1.toChecksumAddress)(`0x${output.slice(24)}`);
            case 'uint256':
            case 'uint120':
                return (0, tiny_big_1.tinyBig)((0, hex_to_decimal_1.hexToDecimal)(`0x${output}`));
            case 'bytes32':
                return `0x${output}`;
            case 'uint8':
                return Number((0, hex_to_decimal_1.hexToDecimal)(`0x${output}`));
            default:
                throw new Error(`essential-eth does not yet support "${outputType}" outputs. Make a PR today!"`);
        }
    });
    return outputs.length === 1 ? outputs[0] : outputs;
}
exports.decodeRPCResponse = decodeRPCResponse;
