"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildRPCPostBody = exports.post = exports.buildFetchInit = void 0;
const isomorphic_unfetch_1 = __importDefault(require("isomorphic-unfetch"));
/**
 * Forms the init field for http fetching
 *
 * @param body
 * @internal
 */
function buildFetchInit(body) {
    return {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
        },
        body: JSON.stringify(body),
    };
}
exports.buildFetchInit = buildFetchInit;
/**
 * Makes a post request with the specified JSON data, normally to the a Ethereum JSON RPC API endpoint
 *
 * @internal
 * @param url the URL to send the request to
 * @param body JSON POST body
 * @returns the JSON response from the server
 * @example
 * ```javascript
 * post('https://free-eth-node.com/api/eth', { jsonrpc: '2.0', id: 1, method: 'eth_gasPrice', params: [] });
 * // '0x66fa8dbfd'
 *
 * post('https://free-eth-node.com/api/eth', { jsonrpc: '2.0', id: 1, method: 'eth_getBalance', params: [ '0x4a986a6dCA6dbf99bC3d17F8D71aFb0d60e740f8', 'latest' ] });
 * // '0x312faeb995df61d4'
 * ```
 */
function post(url, body) {
    return (0, isomorphic_unfetch_1.default)(url, buildFetchInit(body))
        .then(async (r) => {
        const t = await r.text();
        try {
            return JSON.parse(t);
        }
        catch {
            throw new Error(`Invalid JSON RPC response: "${t}"`);
        }
    })
        .then((response) => {
        const result = response?.result;
        if (!result) {
            throw new Error(`Invalid JSON RPC response: ${JSON.stringify(response)}`);
        }
        return response.result;
    });
}
exports.post = post;
/**
 * Prepares data to be sent using the {@link post} function. Data is prepared per the {@link https://en.wikipedia.org/wiki/JSON-RPC#Examples JSON RPC v2 spec}
 *
 * @internal
 * @param method the RPC method to be invoked
 * @param params the parameters to be passed to the defined method
 * @returns a POST method body matching the {@link https://en.wikipedia.org/wiki/JSON-RPC#Examples JSON RPC v2 spec}
 * @example
 * ```javascript
 * buildRPCPostBody('eth_gasPrice', []);
 * // { jsonrpc: '2.0', id: 1, method: 'eth_gasPrice', params: [] }
 *
 * buildRPCPostBody('eth_getBalance', ['0x407d73d8a49eeb85d32cf465507dd71d507100c1', 'latest']);
 * // { jsonrpc: '2.0', id: 1, method: 'eth_getBalance', params: [ '0x4a986a6dCA6dbf99bC3d17F8D71aFb0d60e740f8', 'latest' ] }
 * ```
 */
function buildRPCPostBody(method, params) {
    return {
        jsonrpc: '2.0',
        // TODO: Increment ID will be needed when websocket support is added
        id: 1,
        method,
        params,
    };
}
exports.buildRPCPostBody = buildRPCPostBody;
