/**
 * Converts arrays with types and values into a hex string that can be hashed
 *
 * @param types array of Solidity types, where `type[0]` is the type for `value[0]`
 * @param values array of values, where `value[0]` is of type `type[0]`
 * @returns a hex string with the data given, packed to include its types
 * @example
 * ```javascript
 * const types = ['bool', 'string', 'uint64'];
 * const values = [true, 'text', 30];
 * pack(types, values);
 * // '0x0174657874000000000000001e'
 * ```
 */
export declare function pack(types: ReadonlyArray<string>, values: ReadonlyArray<any>): string;
/**
 * Hashes data from Solidity using the Keccak256 algorithm.
 *
 * Similar to ["solidityKeccak256" in ethers.js](https://docs.ethers.io/v5/api/utils/hashing/#utils-solidityKeccak256)
 *
 * @param types Each [Solidity type](https://docs.soliditylang.org/en/v0.8.13/types.html) corresponding to the values passed in. Helps the function parse and pack data properly.
 * @param values Data to be concatenated (combined) and then hashed.
 * @returns A Keccak256 hash (hex string) based on the values provided
 * @example
 * ```javascript
 * const types = ['string', 'bool', 'uint32'];
 * const values = ['essential-eth is great', true, 14];
 * solidityKeccak256(types, values);
 * // '0xe4d4c8e809faac09d58f468f0aeab9474fe8965d554c6c0f868c433c3fd6acab'
 * ```
 * @example
 * ```javascript
 * const types = ['bytes4', 'uint32[5]'];
 * const values = [[116, 101, 115, 116], [5, 3, 4, 9, 18]];
 * solidityKeccak256(types, values);
 * // '0x038707a887f09355dc545412b058e7ba8f3c74047050c7c5e5e52eec608053d9'
 * ```
 */
export declare function solidityKeccak256(types: ReadonlyArray<string>, values: ReadonlyArray<any>): string;
