"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.weiToEther = void 0;
const tiny_big_1 = require("../shared/tiny-big/tiny-big");
const validate_type_1 = require("../shared/validate-type");
/**
 * Convert from Wei to Ether
 *
 * Similar to ["formatEther" in ethers.js](https://docs.ethers.io/v5/api/utils/display-logic/#utils-formatEther)
 *
 * Similar to ["fromWei" in web3.js](https://web3js.readthedocs.io/en/v1.7.1/web3-utils.html#fromwei)
 *
 * @param weiQuantity the amount of wei to convert to ether
 * @returns a number of ether equivalent to the specified wei
 * @example
 * ```javascript
 * weiToEther('1000000000000000000000').toString()
 * // '1000'
 * weiToEther(1000000000000000000000).toString()
 * // '1000'
 * ```
 * @example
 * ```javascript
 * weiToEther('1000000000000000000000').toNumber()
 * // 1000
 * weiToEther(1000000000000000000000).toNumber()
 * // 1000
 * ```
 */
function weiToEther(weiQuantity) {
    (0, validate_type_1.validateType)(weiQuantity, ['string', 'number', 'object']);
    // eslint-disable-next-line no-useless-catch
    try {
        let _weiQuantity = weiQuantity;
        if (typeof weiQuantity === 'string' && weiQuantity.slice(0, 2) === '0x') {
            _weiQuantity = BigInt(weiQuantity).toString();
        }
        const result = (0, tiny_big_1.tinyBig)(_weiQuantity).div('1000000000000000000');
        return (0, tiny_big_1.tinyBig)(result);
    }
    catch (error) {
        throw error;
    }
}
exports.weiToEther = weiToEther;
