import { cleanBlock } from '../classes/utils/clean-block';
import { cleanLog } from '../classes/utils/clean-log';
import { cleanTransaction } from '../classes/utils/clean-transaction';
import { cleanTransactionReceipt } from '../classes/utils/clean-transaction-receipt';
import { buildRPCPostBody, post } from '../classes/utils/fetchers';
import { hexToDecimal } from '../classes/utils/hex-to-decimal';
import { prepareTransaction } from '../classes/utils/prepare-transaction';
import { logger } from '../logger/logger';
import { tinyBig } from '../shared/tiny-big/tiny-big';
import chainsInfo from './utils/chains-info';
function prepBlockTag(blockTag) {
    return typeof blockTag === 'number'
        ? tinyBig(blockTag).toHexString()
        : blockTag;
}
export class BaseProvider {
    _rpcUrls = [];
    _post = (body) => post(this.selectRpcUrl(), body);
    constructor(rpcUrls) {
        this._rpcUrls = rpcUrls;
    }
    async getNetwork() {
        const hexChainId = (await this.post(buildRPCPostBody('eth_chainId', [])));
        const chainId = hexToDecimal(hexChainId);
        const info = chainsInfo[chainId];
        return {
            chainId: Number(chainId),
            name: info[0] || 'unknown',
            ensAddress: info[1] || null,
        };
    }
    async getBlockNumber() {
        const currentBlockNumber = (await this.post(buildRPCPostBody('eth_blockNumber', [])));
        return Number(hexToDecimal(currentBlockNumber));
    }
    async getTransaction(transactionHash) {
        const [rpcTransaction, blockNumber] = await Promise.all([
            this.post(buildRPCPostBody('eth_getTransactionByHash', [transactionHash])),
            this.getBlock('latest'),
        ]);
        const cleanedTransaction = cleanTransaction(rpcTransaction);
        cleanedTransaction.confirmations =
            blockNumber.number - cleanedTransaction.blockNumber + 1;
        return cleanedTransaction;
    }
    async getTransactionReceipt(transactionHash) {
        const [rpcTransaction, blockNumber] = await Promise.all([
            this.post(buildRPCPostBody('eth_getTransactionReceipt', [transactionHash])),
            this.getBlock('latest'),
        ]);
        const cleanedTransactionReceipt = cleanTransactionReceipt(rpcTransaction);
        cleanedTransactionReceipt.confirmations =
            blockNumber.number - cleanedTransactionReceipt.blockNumber + 1;
        return cleanedTransactionReceipt;
    }
    async getTransactionCount(address, blockTag = 'latest') {
        blockTag = prepBlockTag(blockTag);
        const transactionCount = (await this.post(buildRPCPostBody('eth_getTransactionCount', [address, blockTag])));
        return Number(hexToDecimal(transactionCount));
    }
    async getBlock(timeFrame = 'latest', returnTransactionObjects = false) {
        let type = 'Number';
        if (typeof timeFrame === 'string' && timeFrame.length === 66) {
            type = 'Hash';
        }
        else {
            timeFrame = prepBlockTag(timeFrame);
        }
        const rpcBlock = (await this.post(buildRPCPostBody(`eth_getBlockBy${type}`, [
            timeFrame,
            returnTransactionObjects,
        ])));
        return cleanBlock(rpcBlock, returnTransactionObjects);
    }
    async getGasPrice() {
        const hexGasPrice = (await this.post(buildRPCPostBody('eth_gasPrice', [])));
        return tinyBig(hexToDecimal(hexGasPrice));
    }
    async getBalance(address, blockTag = 'latest') {
        blockTag = prepBlockTag(blockTag);
        const hexBalance = (await this.post(buildRPCPostBody('eth_getBalance', [address, blockTag])));
        return tinyBig(hexToDecimal(hexBalance));
    }
    async getCode(address, blockTag = 'latest') {
        blockTag = prepBlockTag(blockTag);
        const contractCode = (await this.post(buildRPCPostBody('eth_getCode', [address, blockTag])));
        return contractCode;
    }
    async estimateGas(transaction) {
        const rpcTransaction = prepareTransaction(transaction);
        const gasUsed = (await this.post(buildRPCPostBody('eth_estimateGas', [rpcTransaction])));
        return tinyBig(hexToDecimal(gasUsed));
    }
    async getFeeData() {
        const [block, gasPrice] = await Promise.all([
            this.getBlock('latest'),
            this.getGasPrice(),
        ]);
        let lastBaseFeePerGas = null, maxFeePerGas = null, maxPriorityFeePerGas = null;
        if (block && block.baseFeePerGas) {
            lastBaseFeePerGas = block.baseFeePerGas;
            maxPriorityFeePerGas = tinyBig('1500000000');
            maxFeePerGas = tinyBig(block.baseFeePerGas.mul(2).add(maxPriorityFeePerGas));
        }
        return { lastBaseFeePerGas, maxFeePerGas, maxPriorityFeePerGas, gasPrice };
    }
    async getLogs(filter) {
        const filterByRange = filter;
        if (filterByRange.fromBlock)
            filterByRange.fromBlock = prepBlockTag(filterByRange.fromBlock);
        if (filterByRange.toBlock)
            filterByRange.toBlock = prepBlockTag(filterByRange.toBlock);
        const rpcLogs = (await this.post(buildRPCPostBody('eth_getLogs', [filter])));
        const logs = rpcLogs.map((log) => cleanLog(log, false));
        return logs;
    }
    async call(transaction, blockTag = 'latest') {
        if (transaction.gasPrice &&
            (transaction.maxPriorityFeePerGas || transaction.maxFeePerGas)) {
            logger.throwError('Cannot specify both "gasPrice" and ("maxPriorityFeePerGas" or "maxFeePerGas")', {
                gasPrice: transaction.gasPrice,
                maxFeePerGas: transaction.maxFeePerGas,
                maxPriorityFeePerGas: transaction.maxPriorityFeePerGas,
            });
        }
        if (transaction.maxFeePerGas && transaction.maxPriorityFeePerGas) {
            logger.throwError('Cannot specify both "maxFeePerGas" and "maxPriorityFeePerGas"', {
                maxFeePerGas: transaction.maxFeePerGas,
                maxPriorityFeePerGas: transaction.maxPriorityFeePerGas,
            });
        }
        blockTag = prepBlockTag(blockTag);
        const rpcTransaction = prepareTransaction(transaction);
        const transactionRes = (await this.post(buildRPCPostBody('eth_call', [rpcTransaction, blockTag])));
        return transactionRes;
    }
}
